addpath ../dal/

S=importdata('journal.pbio.0030002.sd001.xls');

Ytmp=cell2mat(foreach(@isequal,S.textdata.Sheet1(2:end,1),[],'good'));

Gene=S.textdata.Sheet1(1,7:end-6);

id_sbj=S.data.Sheet1(:,1);  [tmp,I]=unique(id_sbj); code=id_sbj(sort(I));

Ttmp=S.textdata.Sheet1(2:end,3);
Xtmp=S.data.Sheet1(:,6:end-6); 

% Clean up the data
ix = [153, 211];
Xtmp(ix,:)=[]; id_sbj(ix)=[]; Ytmp(ix)=[]; Ttmp(ix)=[];

Xtmp(isnan(Xtmp))=0;


% Collect subjects into cells
Y=zeros(1,length(code));
X=cell(1,length(code));
T=cell(1,length(code));
L=zeros(1,length(code));
for ii=1:length(code)
  I=find(id_sbj==code(ii));
  Y(ii)=mean(Ytmp(I));
  X{ii}=Xtmp(I,:);
  T{ii}=Ttmp(I);
  L(ii)=length(I);
  if ~isequal(T{ii}{1},'t0')
    display(I);
    display(T{ii});
    warning('Index seems to be wrong!');
  end
end

%% Perform cross-validation
xTrials = [100 4];
lambda=exp(linspace(log(10),log(0.001),20));
[Am, As,divTr,divTe,output,memo]=xvalidation(X,Y,'dallrgl',lambda,xTrials,'preproc','meandiffs');

%% Plot accuracy
figure, h=errorbar(log(lambda), Am, As);
set(gca,'fontsize',16);
set(h,'linewidth',2);
grid on;
logticks('x');
set(gca,'position',[0.13, 0.2 0.775, 0.75])
xlabel('Regularization constant');
ylabel('Accuracy');
format_ticks(gca,get(gca,'xticklabel'))
xlim(log([0.0009 11]))


hold on;
plot(xlim,0.8705*ones(1,2),'--','color',[0 .5 0],'linewidth',2);

legend('DALGL','Borgwardt et al.')


%% Plot feature weights at lambda=0.5456
ds=zeros(prod(xTrials), length(Gene));
for kk=1:prod(xTrials)
  [ii,jj]=ind2sub(xTrials,kk);
  ds(kk,:)=sqrt(sum(memo(ii,jj,7).C.w.^2));
end
figure, plot(mean(ds),'linewidth',2);
set(gca,'fontsize',12)                 
set(gca,'position',[0.05 0.2 0.9 0.7]) 
set(gca,'xtick',1:length(Gene))                   
format_ticks(gca,Gene,[],[],[],45);
grid on;
title('Feature weights')
set(gcf,'paperpositionmode','auto')   
set(gcf,'position',[-595, 402, 1591, 263]);